 /* Copyright (c) 2009 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is confidential property of Nordic
 * Semiconductor ASA.Terms and conditions of usage are described in detail
 * in NORDIC SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

#ifndef __HAL_TWI_H__
#define __HAL_TWI_H__

#include <stdbool.h>
#include <stdint.h>

#include "nrf51.h"
#include "nrf51_bitfields.h"

/** @file
* @brief 
*
*
* @defgroup hal_twi Software controlled TWI Master driver
* @{
* @ingroup nrfready_hal
* @brief Software controlled TWI Master driver.
*
* Supported features:
* - Repeated start
* - No multi-master
* - Only 7-bit addressing
* - Supports clock stretching (with optional SMBus style slave timeout)
* - Tries to handle slaves stuck in the middle of transfer
*/

#define TWI_READ_BIT        (0x01)        //!< If this bit is set in the address field, transfer direction is from slave to master.
#define TWI_ISSUE_STOP      ((bool)true)  //!< Parameter for @ref twi_master_transfer
#define TWI_DONT_ISSUE_STOP ((bool)false) //!< Parameter for @ref twi_master_transfer

/**@brief Possible data rates */
typedef enum
{
    hal_twi_freq_100khz = TWI_FREQUENCY_FREQUENCY_K100,
    hal_twi_freq_250khz = TWI_FREQUENCY_FREQUENCY_K250,
    hal_twi_freq_400khz = TWI_FREQUENCY_FREQUENCY_K400
} hal_twi_freq_t;

/**@brief Initialization struct */
typedef struct
{
    uint8_t        scl_pin;   /** Clock signal */
    uint8_t        sda_pin;   /** Data signal */
    hal_twi_freq_t frequency; /** Data rate */
    uint8_t        ppi_chn;   /** Which PPI channel to use @note SoftDevice will use channels 7-15 if enabled. */
} hal_twi_init_t;


/**
 * Initializes TWI bus IO pins and checks the bus is operational.
 *
 * Both pins are configured as Standard-0, No-drive-1 (open drain).
 *
 * @return
 * @retval NRF_SUCCESS        TWI bus is clear for transfers.
 * @retval NRF_ERROR_INTERNAL TWI bus is stuck.
 */
uint32_t hal_twi_init(const hal_twi_init_t* p_params);

/**
 * Transfers data over TWI bus.
 *
 * If TWI master detects even one NACK from the slave or timeout occurs, STOP condition is issued
 * and the function returns false.
 * Bit 0 (@ref TWI_READ_BIT) in the address parameter controls transfer direction;
 * - If 1, master reads data_length number of bytes from the slave
 * - If 0, master writes data_length number of bytes to the slave.
 *
 * @note Make sure at least data_length number of bytes is allocated in data if TWI_READ_BIT is set.
 * @note @ref TWI_ISSUE_STOP
 *
 * @param address Data transfer direction (LSB) / Slave address (7 MSBs).
 * @param data Pointer to data.
 * @param data_length Number of bytes to transfer.
 * @param issue_stop_condition If @ref TWI_ISSUE_STOP, STOP condition is issued before exiting function. If @ref TWI_DONT_ISSUE_STOP, STOP condition is not issued before exiting function. If transfer failed for any reason, STOP condition will be issued in any case.
 * @return true if transfer succeeded
 */
bool hal_twi_transfer(uint8_t address, uint8_t *data, uint8_t data_length, bool issue_stop_condition);

/**
 *@}
 **/

#endif /* __HAL_TWI_H__ */
